@@ -48,6 +48,7 @@
 #include <map>
 using namespace std;
 
+static Chuck_Hid_Driver default_drivers[CK_HID_DEV_COUNT];
 
 struct PhyHidDevIn
 {
@@ -55,7 +56,10 @@ public:
     PhyHidDevIn();
     ~PhyHidDevIn();
     t_CKBOOL open( t_CKINT type, t_CKUINT number );
+    t_CKBOOL read( t_CKINT element_type, t_CKINT element_num, HidMsg * msg );
+    t_CKBOOL send( const HidMsg * msg );
     t_CKBOOL close();
+    std::string name();
     
     t_CKBOOL query_element( HidMsg * query );
 
@@ -82,6 +86,7 @@ struct PhyHidDevOut
 //-----------------------------------------------------------------------------
 // global variables
 //-----------------------------------------------------------------------------
+// per-physical device buffer size
 #define BUFFER_SIZE 8192
 
 std::vector< std::vector<PhyHidDevIn *> > HidInManager::the_matrix;
@@ -130,41 +135,33 @@ t_CKBOOL PhyHidDevIn::open( t_CKINT type, t_CKUINT number )
         EM_log( CK_LOG_WARNING, "PhyHidDevIn: open() failed -> already open!" );
         return FALSE;
     }
-
-    switch( type )
+    
+    if( type <= CK_HID_DEV_NONE || type >= CK_HID_DEV_COUNT )
     {
-        case CK_HID_DEV_JOYSTICK:
-            if( Joystick_open( (int) number ) )
-            {
-                EM_log( CK_LOG_WARNING, "PhyHidDevIn: open() failed -> invalid joystick number %d", number );
-                return FALSE;
-            }            
-                
-            break;
-            
-        case CK_HID_DEV_MOUSE:
-            if( Mouse_open( (int) number ) )
-            {
-                EM_log( CK_LOG_WARNING, "PhyHidDevIn: open() failed -> invalid mouse number %d", number );
-                return FALSE;
-            }
-            
-            break;
-            
-        case CK_HID_DEV_KEYBOARD:
-            if( Keyboard_open( (int) number ) )
-            {
-                EM_log( CK_LOG_WARNING, "PhyHidDevIn: open() failed -> invalid keyboard number %d", number );
-                return FALSE;
-            }
-            
-            break;
-                
-        default:
-            EM_log( CK_LOG_WARNING, "PhyHidDevIn: open operation failed; unknown device-type" );
-            return FALSE;
+        // log
+        EM_log( CK_LOG_WARNING, 
+                "PhyHidDevIn: open() failed -> invalid device type %d", 
+                type );
+        return FALSE;
     }
-
+    
+    if( !default_drivers[type].open )
+    {
+        EM_log( CK_LOG_WARNING, 
+                "PhyHidDevIn: open() failed -> %s does not support open",
+                default_drivers[type].driver_name );
+        return FALSE;
+    }
+    
+    if( default_drivers[type].open( ( int ) number ) )
+    {
+        EM_log( CK_LOG_WARNING, 
+                "PhyHidDevIn: open() failed -> invalid %s number %d",
+                default_drivers[type].driver_name,
+                number );
+        return FALSE;
+    }
+    
     // allocate the buffer
     cbuf = new CBufferAdvance;
     if( !cbuf->initialize( BUFFER_SIZE, sizeof(HidMsg) ) )
@@ -178,10 +175,60 @@ t_CKBOOL PhyHidDevIn::open( t_CKINT type, t_CKUINT number )
     device_type = type;
     device_num = number;
 
-
     return TRUE;
 }
 
+//-----------------------------------------------------------------------------
+// name: read()
+// desc: query specific elements of the hid device
+//-----------------------------------------------------------------------------
+t_CKBOOL PhyHidDevIn::read( t_CKINT element_type, t_CKINT element_num, HidMsg * msg )
+{
+    if( !default_drivers[device_type].read )
+    {
+        EM_log( CK_LOG_WARNING, 
+                "PhyHidDevIn: read() failed -> %s does not support read",
+                default_drivers[device_type].driver_name );
+        return FALSE;
+    }
+    
+    if( default_drivers[device_type].read( device_num, element_type, 
+                                           element_num, msg ) )
+    {
+        EM_log( CK_LOG_WARNING, 
+                "PhyHidDevIn: read() failed for %s %i, element type %i, element number %i",
+                default_drivers[device_type].driver_name, device_num, 
+                element_type, element_num );
+        return FALSE;
+    }
+    
+    return TRUE;
+}
+
+//-----------------------------------------------------------------------------
+// name: send()
+// desc: send message to a HID device
+//-----------------------------------------------------------------------------
+t_CKBOOL PhyHidDevIn::send( const HidMsg * msg )
+{
+    if( !default_drivers[device_type].send )
+    {
+        EM_log( CK_LOG_WARNING, 
+                "PhyHidDevIn: send() failed -> %s does not support send",
+                default_drivers[device_type].driver_name );
+        return FALSE;
+    }
+    
+    if( default_drivers[device_type].send( device_num, msg ) )
+    {
+        EM_log( CK_LOG_WARNING, 
+                "PhyHidDevIn: send() failed for %s %i",
+                default_drivers[device_type].driver_name, device_num );
+        return FALSE;
+    }
+    
+    return TRUE;
+}
 
 //-----------------------------------------------------------------------------
 // name: close()
@@ -197,24 +244,30 @@ t_CKBOOL PhyHidDevIn::close()
         // TODO: release references from cbuf?
     }
 
-    switch( device_type )
+    if( device_type <= CK_HID_DEV_NONE || device_type >= CK_HID_DEV_COUNT )
     {
-        case CK_HID_DEV_JOYSTICK:
-            Joystick_close( device_num );
-                        
-            break;
-            
-        case CK_HID_DEV_MOUSE:
-            Mouse_close( device_num );
-            break;
-            
-        case CK_HID_DEV_KEYBOARD:
-            Keyboard_close( device_num );
-            break;
-            
-        default:
-            EM_log( CK_LOG_WARNING, "PhyHidDevIn: close operation failed; device not opened" );
-            return FALSE;
+        // log
+        EM_log( CK_LOG_WARNING, 
+                "PhyHidDevIn: close() failed -> invalid device type %d", 
+                device_type );
+        return FALSE;
+    }
+    
+    if( !default_drivers[device_type].close )
+    {
+        EM_log( CK_LOG_WARNING, 
+                "PhyHidDevIn: close() failed -> %s does not support close",
+                default_drivers[device_type].driver_name );
+        return FALSE;
+    }
+    
+    if( default_drivers[device_type].close( ( int ) device_num ) )
+    {
+        EM_log( CK_LOG_WARNING, 
+                "PhyHidDevIn: close() failed -> invalid %s number %d",
+                default_drivers[device_type].driver_name,
+                device_num );
+        return FALSE;
     }
     
     device_type = CK_HID_DEV_NONE;
@@ -223,7 +276,40 @@ t_CKBOOL PhyHidDevIn::close()
     return TRUE;
 }
 
-
+//-----------------------------------------------------------------------------
+// name: name()
+// desc: retrieve device name
+//-----------------------------------------------------------------------------
+string PhyHidDevIn::name()
+{
+    if( device_type == CK_HID_DEV_NONE )
+        return " ";
+    
+    if( device_type <= CK_HID_DEV_NONE || device_type >= CK_HID_DEV_COUNT )
+    {
+        // log
+        EM_log( CK_LOG_WARNING, 
+                "PhyHidDevIn: name() failed -> invalid device type %d", 
+                device_type );
+        return " ";
+    }
+    
+    if( !default_drivers[device_type].name )
+        return default_drivers[device_type].driver_name;
+    
+    const char * _name;
+    
+    if( !( _name = default_drivers[device_type].name( ( int ) device_num ) ) )
+    {
+        EM_log( CK_LOG_WARNING, 
+                "PhyHidDevIn: name() failed -> invalid %s number %d",
+                default_drivers[device_type].driver_name,
+                device_num );
+        return " ";
+    }
+    
+    return _name;
+}
 
 //-----------------------------------------------------------------------------
 // name: HidOut()
@@ -358,6 +444,8 @@ void HidInManager::init()
 	EM_pushlog();
     if( has_init == FALSE )
     {
+        init_default_drivers();
+        
         // allocate the matrix
         the_matrix.resize( CK_HID_DEV_COUNT );
         // resize each vector
@@ -370,59 +458,114 @@ void HidInManager::init()
         msg_buffer = new CBufferSimple;
         msg_buffer->initialize( 1000, sizeof( HidMsg ) );
         
+#ifndef __MACOSX_CORE__
         Hid_init();
+#endif // __MACOSX_CORE__
+        
+        for( size_t j = 0; j < CK_HID_DEV_COUNT; j++ )
+        {
+            if( default_drivers[j].init )
+                default_drivers[j].init();
+        }
         
-        Joystick_init();
-        Mouse_init();
-        Keyboard_init();
-
         has_init = TRUE;
     }
 
 	EM_poplog();
 }
 
+void HidInManager::init_default_drivers()
+{
+    memset( default_drivers, 0, CK_HID_DEV_COUNT * sizeof( Chuck_Hid_Driver ) );
+    
+    default_drivers[CK_HID_DEV_JOYSTICK].init = Joystick_init;
+    default_drivers[CK_HID_DEV_JOYSTICK].quit = Joystick_quit;
+    default_drivers[CK_HID_DEV_JOYSTICK].count = Joystick_count;
+    default_drivers[CK_HID_DEV_JOYSTICK].open = Joystick_open;
+    default_drivers[CK_HID_DEV_JOYSTICK].close = Joystick_close;
+    default_drivers[CK_HID_DEV_JOYSTICK].name = Joystick_name;
+    default_drivers[CK_HID_DEV_JOYSTICK].driver_name = "joystick";
+    
+    default_drivers[CK_HID_DEV_MOUSE].init = Mouse_init;
+    default_drivers[CK_HID_DEV_MOUSE].quit = Mouse_quit;
+    default_drivers[CK_HID_DEV_MOUSE].count = Mouse_count;
+    default_drivers[CK_HID_DEV_MOUSE].open = Mouse_open;
+    default_drivers[CK_HID_DEV_MOUSE].close = Mouse_close;
+    default_drivers[CK_HID_DEV_MOUSE].name = Mouse_name;
+    default_drivers[CK_HID_DEV_MOUSE].driver_name = "mouse";
+    
+    default_drivers[CK_HID_DEV_KEYBOARD].init = Keyboard_init;
+    default_drivers[CK_HID_DEV_KEYBOARD].quit = Keyboard_quit;
+    default_drivers[CK_HID_DEV_KEYBOARD].count = Keyboard_count;
+    default_drivers[CK_HID_DEV_KEYBOARD].open = Keyboard_open;
+    default_drivers[CK_HID_DEV_KEYBOARD].close = Keyboard_close;
+    default_drivers[CK_HID_DEV_KEYBOARD].send = Keyboard_send;
+    default_drivers[CK_HID_DEV_KEYBOARD].name = Keyboard_name;
+    default_drivers[CK_HID_DEV_KEYBOARD].driver_name = "keyboard";
+    
+    default_drivers[CK_HID_DEV_WIIREMOTE].init = WiiRemote_init;
+    default_drivers[CK_HID_DEV_WIIREMOTE].quit = WiiRemote_quit;
+    default_drivers[CK_HID_DEV_WIIREMOTE].count = WiiRemote_count;
+    default_drivers[CK_HID_DEV_WIIREMOTE].open = WiiRemote_open;
+    default_drivers[CK_HID_DEV_WIIREMOTE].close = WiiRemote_close;
+    default_drivers[CK_HID_DEV_WIIREMOTE].send = WiiRemote_send;
+    default_drivers[CK_HID_DEV_WIIREMOTE].name = WiiRemote_name;
+    default_drivers[CK_HID_DEV_WIIREMOTE].driver_name = "wii remote";
+    
+    default_drivers[CK_HID_DEV_TILTSENSOR].init = TiltSensor_init;
+    default_drivers[CK_HID_DEV_TILTSENSOR].quit = TiltSensor_quit;
+    default_drivers[CK_HID_DEV_TILTSENSOR].count = TiltSensor_count;
+    default_drivers[CK_HID_DEV_TILTSENSOR].open = TiltSensor_open;
+    default_drivers[CK_HID_DEV_TILTSENSOR].close = TiltSensor_close;
+    default_drivers[CK_HID_DEV_TILTSENSOR].read = TiltSensor_read;
+    default_drivers[CK_HID_DEV_TILTSENSOR].name = TiltSensor_name;
+    default_drivers[CK_HID_DEV_TILTSENSOR].driver_name = "tilt sensor";
+}
 
 void HidInManager::cleanup()
 {
     // log
     EM_log( CK_LOG_INFO, "shutting down HID..." );
-
-    /*
-    // flag
-    thread_going = FALSE;
-     
-    // break Hid_wait();
-    Hid_quit();
-
-	// clean up
-    if( the_thread != NULL )
-        SAFE_DELETE( the_thread );
-
-	// clean up subsystems
-    Joystick_quit();
-    Mouse_quit();
-    Keyboard_quit();
-
-    if( msg_buffer )
-    {
-        msg_buffer->cleanup();
-        SAFE_DELETE( msg_buffer );
-    }
-     
-    // init
-    has_init = FALSE;
-    */
-
-    // loop
-    for( vector<vector<PhyHidDevIn *> >::size_type i = 0; i < the_matrix.size(); i++ )
-    {
+    
+    if( has_init )
+    {    
         // loop
-        for( vector<PhyHidDevIn *>::size_type j = 0; j < the_matrix[i].size(); j++ )
+        for( vector<vector<PhyHidDevIn *> >::size_type i = 0; i < the_matrix.size(); i++ )
         {
-            // deallocate devices
-            SAFE_DELETE( the_matrix[i][j] );
+            // loop
+            for( vector<PhyHidDevIn *>::size_type j = 0; j < the_matrix[i].size(); j++ )
+            {
+                // deallocate devices
+                SAFE_DELETE( the_matrix[i][j] );
+            }
+        }    
+        
+        // flag
+        thread_going = FALSE;
+        
+        // break Hid_poll();
+        Hid_quit();
+        
+        // clean up
+        if( the_thread != NULL )
+            SAFE_DELETE( the_thread );
+        
+        // clean up subsystems
+        for( size_t j = 0; j < CK_HID_DEV_COUNT; j++ )
+        {
+            if( default_drivers[j].quit )
+                default_drivers[j].quit();
         }
+        
+        if( msg_buffer )
+        {
+            msg_buffer->cleanup();
+            SAFE_DELETE( msg_buffer );
+        }
+        
+        // init
+        has_init = FALSE;
+        //*/
     }
 }
 
@@ -443,7 +586,18 @@ t_CKBOOL HidInManager::open( HidIn * hin, t_CKINT device_type, t_CKINT device_nu
             device_type );
         return FALSE;
     }
-
+    
+    // start thread
+    if( the_thread == NULL )
+    {
+        // allocate
+        the_thread = new XThread;
+        // flag
+        thread_going = TRUE;
+        // start
+        the_thread->start( cb_hid_input, NULL );
+    }
+    
     // get the vector
     vector<PhyHidDevIn *> & v = the_matrix[device_type];
 
@@ -456,7 +610,9 @@ t_CKBOOL HidInManager::open( HidIn * hin, t_CKINT device_type, t_CKINT device_nu
         if( !phin->open( device_type, device_num ) )
         {
             // log
-            EM_error2( 0, "HidIn: couldn't open HID device %d:%d...", device_type, device_num );
+            // should this use EM_log instead, with a higher log level?
+            EM_error2( 0, "HidIn: couldn't open %s %d...", 
+                       default_drivers[device_type].driver_name, device_num );
             SAFE_DELETE( phin );
             return FALSE;
         }
@@ -482,24 +638,10 @@ t_CKBOOL HidInManager::open( HidIn * hin, t_CKINT device_type, t_CKINT device_nu
     hin->m_read_index = hin->m_buffer->join( (Chuck_Event *)hin->SELF );
     hin->m_device_num = (t_CKUINT)device_num;
 
-    // start thread
-    if( the_thread == NULL )
-    {
-        // allocate
-        the_thread = new XThread;
-        // flag
-        thread_going = TRUE;
-        // start
-        the_thread->start( cb_hid_input, NULL );
-    }
-
     // done
     return TRUE;
 }
 
-
-
-
 //-----------------------------------------------------------------------------
 // name: close()
 // desc: close
@@ -510,16 +652,13 @@ t_CKBOOL HidIn::close()
         return FALSE;
 
     // close
-    // HidInManager::close( this );
+    //HidInManager::close( this );
 
     m_valid = FALSE;
 
     return TRUE;
 }
 
-
-
-
 //-----------------------------------------------------------------------------
 // name: empty()
 // desc: is empty?
@@ -534,8 +673,8 @@ t_CKBOOL HidIn::empty()
 
 
 //-----------------------------------------------------------------------------
-// name: get()
-// desc: get message
+// name: recv()
+// desc: receive message
 //-----------------------------------------------------------------------------
 t_CKUINT HidIn::recv( HidMsg * msg )
 {
@@ -543,6 +682,42 @@ t_CKUINT HidIn::recv( HidMsg * msg )
     return m_buffer->get( msg, 1, m_read_index );
 }
 
+//-----------------------------------------------------------------------------
+// name: read()
+// desc: read
+//-----------------------------------------------------------------------------
+t_CKBOOL HidIn::read( t_CKINT type, t_CKINT num, HidMsg * msg )
+{
+    if( !m_valid || !phin )
+        return FALSE;
+    
+    // do read
+    return phin->read( type, num, msg );
+}
+
+//-----------------------------------------------------------------------------
+// name: send()
+// desc: send
+//-----------------------------------------------------------------------------
+t_CKBOOL HidIn::send( const HidMsg * msg )
+{
+    if( !m_valid || !phin )
+        return FALSE;
+    
+    // do send
+    return phin->send( msg );
+}
+
+//-----------------------------------------------------------------------------
+// name: name()
+// desc: get device name
+//-----------------------------------------------------------------------------
+std::string HidIn::name()
+{
+    if( m_valid && phin )
+        return phin->name();
+    return " ";
+}
 
 //-----------------------------------------------------------------------------
 // name: cb_hid_input
@@ -575,7 +750,9 @@ void * HidInManager::cb_hid_input( void * stuff )
 unsigned __stdcall HidInManager::cb_hid_input( void * stuff )
 #endif 
 {
-    HidMsg msg;
+#ifdef __MACOSX_CORE__
+    Hid_init();
+#endif
 
     // keep going
     while( thread_going )
@@ -593,24 +770,42 @@ unsigned __stdcall HidInManager::cb_hid_input( void * stuff )
 // name: probeHidIn()
 // desc: ...
 //-----------------------------------------------------------------------------
-void probeHidIn()
+void HidInManager::probeHidIn()
 {
+    if( !has_init )
+    {
+        for( size_t i = 0; i < CK_HID_DEV_COUNT; i++ )
+        {
+            if( default_drivers[i].init && default_drivers[i].probe )
+                default_drivers[i].init();
+        }
+    }
+    
+    for( size_t i = 0; i < CK_HID_DEV_COUNT; i++ )
+    {
+        if( default_drivers[i].probe )
+            default_drivers[i].probe();
+    }
+    
+    if( !has_init )
+    {
+        for( size_t i = 0; i < CK_HID_DEV_COUNT; i++ )
+        {
+            if( default_drivers[i].quit )
+                default_drivers[i].quit();
+        }
+    }
 }
 
-
-
-
 //-----------------------------------------------------------------------------
 // name: probeHidOut()
 // desc: ...
 //-----------------------------------------------------------------------------
-void probeHidOut()
+void HidInManager::probeHidOut()
 {
+    
 }
 
-
-
-
 HidOutManager::HidOutManager()
 {
     the_phouts.resize( 1024 );
